@@ -35,19 +35,43 @@ vector_alloc(void)
 }
 
 /* allocated one slot */
-void *
+bool
 vector_alloc_slot(vector v)
 {
-	v->allocated += VECTOR_DEFAULT_SIZE;
-	if (v->slot)
-		v->slot = REALLOC(v->slot, sizeof (void *) * v->allocated);
-	else
-		v->slot = (void *) MALLOC(sizeof (void *) * v->allocated);
+	void *new_slot = NULL;
+	int new_allocated;
+	int i;
 
-	if (!v->slot)
-		v->allocated -= VECTOR_DEFAULT_SIZE;
+	if (!v)
+		return false;
 
-	return v->slot;
+	new_allocated = v->allocated + VECTOR_DEFAULT_SIZE;
+	new_slot = REALLOC(v->slot, sizeof (void *) * new_allocated);
+	if (!new_slot)
+		return false;
+
+	v->slot = new_slot;
+	for (i = v->allocated; i < new_allocated; i++)
+		v->slot[i] = NULL;
+
+	v->allocated = new_allocated;
+	return true;
+}
+
+int
+vector_move_up(vector v, int src, int dest)
+{
+	void *value;
+	int i;
+	if (dest == src)
+		return 0;
+	if (dest > src || src >= v->allocated)
+		return -1;
+	value = v->slot[src];
+	for (i = src - 1; i >= dest; i--)
+		v->slot[i + 1] = v->slot[i];
+	v->slot[dest] = value;
+	return 0;
 }
 
 void *
@@ -58,7 +82,7 @@ vector_insert_slot(vector v, int slot, void *value)
 	if (!vector_alloc_slot(v))
 		return NULL;
 
-	for (i = (v->allocated /VECTOR_DEFAULT_SIZE) - 2; i >= slot; i--)
+	for (i = VECTOR_SIZE(v) - 2; i >= slot; i--)
 		v->slot[i + 1] = v->slot[i];
 
 	v->slot[slot] = value;
@@ -71,7 +95,10 @@ find_slot(vector v, void * addr)
 {
 	int i;
 
-	for (i = 0; i < (v->allocated / VECTOR_DEFAULT_SIZE); i++)
+	if (!v)
+		return -1;
+
+	for (i = 0; i < VECTOR_SIZE(v); i++)
 		if (v->slot[i] == addr)
 			return i;
 
@@ -83,20 +110,27 @@ vector_del_slot(vector v, int slot)
 {
 	int i;
 
-	if (!v->allocated || slot < 0 || slot > VECTOR_SIZE(v))
+	if (!v || !v->allocated || slot < 0 || slot >= VECTOR_SIZE(v))
 		return;
 
-	for (i = slot + 1; i < (v->allocated / VECTOR_DEFAULT_SIZE); i++)
+	for (i = slot + 1; i < VECTOR_SIZE(v); i++)
 		v->slot[i-1] = v->slot[i];
 
 	v->allocated -= VECTOR_DEFAULT_SIZE;
 
-	if (!v->allocated) {
+	if (v->allocated <= 0) {
 		FREE(v->slot);
 		v->slot = NULL;
+		v->allocated = 0;
+	} else {
+		void *new_slot;
+
+		new_slot = REALLOC(v->slot, sizeof (void *) * v->allocated);
+		if (!new_slot)
+			v->allocated += VECTOR_DEFAULT_SIZE;
+		else
+			v->slot = new_slot;
 	}
-	else
-		v = REALLOC(v->slot, sizeof (void *) * v->allocated);
 }
 
 void
@@ -104,24 +138,34 @@ vector_repack(vector v)
 {
 	int i;
 
-	if (!v->allocated)
+	if (!v || !v->allocated)
 		return;
 
-	for (i = 0; i < (v->allocated / VECTOR_DEFAULT_SIZE); i++)
+	for (i = 0; i < VECTOR_SIZE(v); i++)
 		if (i > 0 && v->slot[i] == NULL)
 			vector_del_slot(v, i--);
 }
 
+vector
+vector_reset(vector v)
+{
+	if (!v)
+		return NULL;
+
+	if (v->slot)
+		FREE(v->slot);
+
+	v->allocated = 0;
+	v->slot = NULL;
+	return v;
+}
+
 /* Free memory vector allocation */
 void
 vector_free(vector v)
 {
-	if (!v)
+	if (!vector_reset(v))
 		return;
-
-	if (v->slot)
-		FREE(v->slot);
-
 	FREE(v);
 }
 
@@ -145,7 +189,23 @@ free_strvec(vector strvec)
 void
 vector_set_slot(vector v, void *value)
 {
-	unsigned int i = v->allocated - 1;
+	unsigned int i;
 
+	if (!v)
+		return;
+
+	i = VECTOR_SIZE(v) - 1;
 	v->slot[i] = value;
 }
+
+int vector_find_or_add_slot(vector v, void *value)
+{
+	int n = find_slot(v, value);
+
+	if (n >= 0)
+		return n;
+	if (!vector_alloc_slot(v))
+		return -1;
+	vector_set_slot(v, value);
+	return VECTOR_SIZE(v) - 1;
+}
